
require 'pp'
require_relative 'hand_iterators'

#
# r_to_hom.rb
# California Jack Cassidy, 2014
#
# for a given relative volatility r
# compute how often hand order is maintained
# in the 0..1 Von Neuman game
#
# Convert for Bernasconi equivalent, the q value, i.e.
# the likelihood of order not maintained.


def hom_calc(r_vol)
  count = 42

  #base_iter = Double_hand_random_iterator.new(1.0, count)
  base_iter = Double_hand_cycle_iterator.new(1.0, count)

  same = 0
  changed = 0
  loop do  # loop over base values

    bases = base_iter.another
    break if bases.nil?

    #add_iter = Double_hand_random_iterator.new(r_vol, count)
    add_iter = Double_hand_cycle_iterator.new(r_vol, count)

    loop do  # loop over add values

      # for each base value, try all the add values
      adds = add_iter.another
      break if adds.nil?

      if bases[0] < bases[1] ==
          bases[0] + adds[0] < bases[1] + adds[1]
        same += 1
      else
        changed += 1
      end

    end  # add loop

  end  # base loop

  return same.to_f / (same + changed)

end

# Main program loop
begin
  loop do
    puts "Enter relative volatility: "
    r_vol = gets.chomp.to_f
    abort("Goodbye") if r_vol == 0.0
    abort("Volatility must be greater than zero") if r_vol < 0.0
    hom = hom_calc(r_vol).round(5)
    puts "HOM = #{hom} so q = #{(1 - hom).round(5)}"
  end

end
